unit uOptions;
{$INCLUDE 'NamedFoldersLite.inc'}

interface

uses
  Windows, SysUtils, Classes, Generics.Collections,
  {$INCLUDE 'DelphiFarUnits.inc'} ,
  uSystem, uLog, uMessages, uConsts, uFunctions, uInfo;

type
  ENamedFolderError = class(Exception);

  TOptions = class(TFarConfig)
  private
    fLastPanelDirectory: string;
    fSaveLastPanelDirectory: boolean;
    fUseSubstitutionEngines: boolean;
    fUseSubstitutionEngine_KnownFolders: boolean;
    fUseSubstitutionEngine_Environment: boolean;
    fUseSubstitutionEngine_BatchEnvironment: boolean;
    fUseSubstitutionEngine_Registry: boolean;
    fAllowPathSuffix: boolean;
    fAllowPluginCalls: boolean;
    fAllowFuzzySearch: boolean;
    fAllowSearchOverDirectories: boolean;
  protected
    function ValidateName(const Name: string): string;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Init;
    procedure Load;
    procedure Save;
    function NamedFolderInfoExists(const Name: string): boolean;
    function LoadNamedFolderInfo(const Name: string; out Info: TNamedFolderInfo): boolean;
    function SaveNamedFolderInfo(Info: TNamedFolderInfo): boolean;
    function DeleteNamedFolderInfo(const Name: string): boolean;
    procedure NamedFoldersList(out List: TStringList);
    property LastPanelDirectory: string read fLastPanelDirectory write fLastPanelDirectory;
    property SaveLastPanelDirectory: boolean read fSaveLastPanelDirectory write fSaveLastPanelDirectory;
    property UseSubstitutionEngines: boolean read fUseSubstitutionEngines write fUseSubstitutionEngines;
    property UseSubstitutionEngine_KnownFolders: boolean read fUseSubstitutionEngine_KnownFolders write fUseSubstitutionEngine_KnownFolders;
    property UseSubstitutionEngine_Environment: boolean read fUseSubstitutionEngine_Environment write fUseSubstitutionEngine_Environment;
    property UseSubstitutionEngine_BatchEnvironment: boolean read fUseSubstitutionEngine_BatchEnvironment write fUseSubstitutionEngine_BatchEnvironment;
    property UseSubstitutionEngine_Registry: boolean read fUseSubstitutionEngine_Registry write fUseSubstitutionEngine_Registry;
    property AllowPathSuffix: boolean read fAllowPathSuffix write fAllowPathSuffix;
    property AllowPluginCalls: boolean read fAllowPluginCalls write fAllowPluginCalls;
    property AllowFuzzySearch: boolean read fAllowFuzzySearch write fAllowFuzzySearch;
    property AllowSearchOverDirectories: boolean read fAllowSearchOverDirectories write fAllowSearchOverDirectories;
  end;

implementation

const
  OPTION_LASTPANELDIRECTORY = 'LastPanelDir';
  DEFAULT_LASTPANELDIRECTORY = '';
  OPTION_SAVELASTPANELDIRECTORY = 'SaveLastPanelDir';
  DEFAULT_SAVELASTPANELDIRECTORY = False;
  OPTION_ALLOWPATHSUFFIX = 'AllowPathSuffix';
  DEFAULT_ALLOWPATHSUFFIX = False;
  OPTION_ALLOWPLUGINCALLS = 'AllowPluginCalls';
  DEFAULT_ALLOWPLUGINCALLS = False;
  OPTION_ALLOWFUZZYSEARCH = 'AllowFuzzySearch';
  DEFAULT_ALLOWFUZZYSEARCH = True;
  OPTION_ALLOWSEARCHOVERDIRECTORIES = 'AllowSearchOverDir';
  DEFAULT_ALLOWSEARCHOVERDIRECTORIES = False;

const
  OPTION_USESUBSTITUTIONENGINES = 'UseSubstEngines';
  DEFAULT_USESUBSTITUTIONENGINES = False;
  OPTION_USESUBSTITUTIONENGINE_KNOWNFOLDERS = 'UseSubstEngine_Known';
  DEFAULT_USESUBSTITUTIONENGINE_KNOWNFOLDERS = False;
  OPTION_USESUBSTITUTIONENGINE_ENVIRONMENT = 'UseSubstEngines_Env';
  DEFAULT_USESUBSTITUTIONENGINE_ENVIRONMENT = False;
  OPTION_USESUBSTITUTIONENGINE_BATCHENVIRONMENT = 'UseSubstEngines_BatchEnv';
  DEFAULT_USESUBSTITUTIONENGINE_BATCHENVIRONMENT = False;
  OPTION_USESUBSTITUTIONENGINE_REGISTRY = 'UseSubstEngines_Reg';
  DEFAULT_USESUBSTITUTIONENGINE_REGISTRY = False;

const
  NAMEDFOLDERS_PATH = '\folders';
  PATHOPTION_DIRECTORY = 'Directory';
  PATHOPTION_PLUGINID = 'PluginID';
  PATHOPTION_PARAM = 'Param';
  PATHOPTION_HOSTFILE = 'HostFile';
  PATHOPTION_DESCRIPTION = 'Description';
  PATHOPTION_FLAGS = 'Flags';

{ TDirSyncOptions }

constructor TOptions.Create;
begin
  inherited Create('NamedFoldersLite', False);
  Init;
  Load;
end;

function TOptions.DeleteNamedFolderInfo(const Name: string): boolean;
var
  FinalName: string;
begin
  Result := False;
  FinalName := ValidateName(Name);
  if Open(NAMEDFOLDERS_PATH, True) then
    try
      Result := DeleteKey(FinalName);
    finally
      Close;
    end;
end;

destructor TOptions.Destroy;
begin
  inherited;
end;

procedure TOptions.Init;
begin
  fLastPanelDirectory := OPTION_LASTPANELDIRECTORY;
  fSaveLastPanelDirectory := DEFAULT_SAVELASTPANELDIRECTORY;
  fUseSubstitutionEngines := DEFAULT_USESUBSTITUTIONENGINES;
  fUseSubstitutionEngine_KnownFolders := DEFAULT_USESUBSTITUTIONENGINE_KNOWNFOLDERS;
  fUseSubstitutionEngine_Environment := DEFAULT_USESUBSTITUTIONENGINE_ENVIRONMENT;
  fUseSubstitutionEngine_BatchEnvironment := DEFAULT_USESUBSTITUTIONENGINE_BATCHENVIRONMENT;
  fUseSubstitutionEngine_Registry := DEFAULT_USESUBSTITUTIONENGINE_REGISTRY;
  fAllowPathSuffix := DEFAULT_ALLOWPATHSUFFIX;
  fAllowPluginCalls := DEFAULT_ALLOWPLUGINCALLS;
  fAllowFuzzySearch := DEFAULT_ALLOWFUZZYSEARCH;
  fAllowSearchOverDirectories := DEFAULT_ALLOWSEARCHOVERDIRECTORIES;
end;

procedure TOptions.Load;
begin
  if Open then
    try
      fLastPanelDirectory := ReadString(OPTION_LASTPANELDIRECTORY, OPTION_LASTPANELDIRECTORY);
      fSaveLastPanelDirectory := ReadBoolean(OPTION_SAVELASTPANELDIRECTORY, DEFAULT_SAVELASTPANELDIRECTORY);
      fUseSubstitutionEngines := ReadBoolean(OPTION_USESUBSTITUTIONENGINES, DEFAULT_USESUBSTITUTIONENGINES);
      fUseSubstitutionEngine_KnownFolders := ReadBoolean(OPTION_USESUBSTITUTIONENGINE_KNOWNFOLDERS, DEFAULT_USESUBSTITUTIONENGINE_KNOWNFOLDERS);
      fUseSubstitutionEngine_Environment := ReadBoolean(OPTION_USESUBSTITUTIONENGINE_ENVIRONMENT, DEFAULT_USESUBSTITUTIONENGINE_ENVIRONMENT);
      fUseSubstitutionEngine_BatchEnvironment := ReadBoolean(OPTION_USESUBSTITUTIONENGINE_BATCHENVIRONMENT, DEFAULT_USESUBSTITUTIONENGINE_BATCHENVIRONMENT);
      fUseSubstitutionEngine_Registry := ReadBoolean(OPTION_USESUBSTITUTIONENGINE_REGISTRY, DEFAULT_USESUBSTITUTIONENGINE_REGISTRY);
      fAllowPathSuffix := ReadBoolean(OPTION_ALLOWPATHSUFFIX, DEFAULT_ALLOWPATHSUFFIX);
      fAllowPluginCalls := ReadBoolean(OPTION_ALLOWPLUGINCALLS, DEFAULT_ALLOWPLUGINCALLS);
      fAllowFuzzySearch := ReadBoolean(OPTION_ALLOWFUZZYSEARCH, DEFAULT_ALLOWFUZZYSEARCH);
      fAllowSearchOverDirectories := ReadBoolean(OPTION_ALLOWSEARCHOVERDIRECTORIES, DEFAULT_ALLOWSEARCHOVERDIRECTORIES);
    finally
      Close;
    end;
end;

function TOptions.LoadNamedFolderInfo(const Name: string; out Info: TNamedFolderInfo): boolean;
var
  Directory, PluginStr, FinalName: string;
begin
  Result := False;
  FinalName := ValidateName(Name);
  if Open(NAMEDFOLDERS_PATH + '\' + FinalName, True) then
    if ReadStringEx(PATHOPTION_DIRECTORY, Directory) then
      try
        Info := TNamedFolderInfo.Create;
        try
          Info.Name := FinalName;
          Info.Directory := Directory;
          if ReadStringEx(PATHOPTION_PLUGINID, PluginStr) then
            if PluginStr <> '' then
              Info.PluginID := StringToGUID(PluginStr);
          Info.Param := ReadString(PATHOPTION_PARAM);
          Info.HostFile := ReadString(PATHOPTION_HOSTFILE);
          Info.Description := ReadString(PATHOPTION_DESCRIPTION);
          Info.Flags := ReadInteger(PATHOPTION_FLAGS);
          Result := True;
        except
          FreeAndNil(Info);
          Raise;
        end;
      finally
        Close;
      end;
end;

function TOptions.NamedFolderInfoExists(const Name: string): boolean;
var
  List: TStringList;
  FinalName: string;
begin
  Result := False;
  FinalName := ValidateName(Name);
  if Open(NAMEDFOLDERS_PATH, True) then
    try
      List := TStringList.Create;
      try
        ReadKeyNames(List);
        if List.IndexOf(FinalName) >= 0 then
          Result := True;
      finally
        FreeAndNil(List);
      end;
    finally
      Close;
    end;
end;

procedure TOptions.NamedFoldersList(out List: TStringList);
begin
  List := TStringList.Create;
  try
    if Open(NAMEDFOLDERS_PATH, True) then
      try
        ReadKeyNames(List);
      finally
        Close;
      end;
  except
    FreeAndNil(List);
    Raise;
  end;
end;

procedure TOptions.Save;
begin
  if Open then
    try
      WriteString(OPTION_LASTPANELDIRECTORY, fLastPanelDirectory);
      WriteBoolean(OPTION_SAVELASTPANELDIRECTORY, fSaveLastPanelDirectory);
      WriteBoolean(OPTION_USESUBSTITUTIONENGINES, fUseSubstitutionEngines);
      WriteBoolean(OPTION_USESUBSTITUTIONENGINE_KNOWNFOLDERS, fUseSubstitutionEngine_KnownFolders);
      WriteBoolean(OPTION_USESUBSTITUTIONENGINE_ENVIRONMENT, fUseSubstitutionEngine_Environment);
      WriteBoolean(OPTION_USESUBSTITUTIONENGINE_BATCHENVIRONMENT, fUseSubstitutionEngine_BatchEnvironment);
      WriteBoolean(OPTION_USESUBSTITUTIONENGINE_REGISTRY, fUseSubstitutionEngine_Registry);
      WriteBoolean(OPTION_ALLOWPATHSUFFIX, fAllowPathSuffix);
      WriteBoolean(OPTION_ALLOWPLUGINCALLS, fAllowPluginCalls);
      WriteBoolean(OPTION_ALLOWFUZZYSEARCH, fAllowFuzzySearch);
      WriteBoolean(OPTION_ALLOWSEARCHOVERDIRECTORIES, fAllowSearchOverDirectories);
    finally
      Close;
    end;
end;

function TOptions.SaveNamedFolderInfo(Info: TNamedFolderInfo): boolean;
var
  FinalName: string;
begin
  Result := False;
  FinalName := ValidateName(Info.Name);
  if Open(NAMEDFOLDERS_PATH + '\' + FinalName) then
    try
      //Log('Saving: "%s", "%s", "%s", "%s"', [Info.Directory, GUIDToString(Info.PluginID), Info.Param, Info.HostFile]);
      WriteString(PATHOPTION_DIRECTORY, Info.Directory);
      if IsEqualGUID(Info.PluginID, NULL_GUID) then
        DeleteValue(PATHOPTION_PLUGINID)
      else
        WriteString(PATHOPTION_PLUGINID, GUIDToString(Info.PluginID));
      WriteString(PATHOPTION_PARAM, Info.Param);
      WriteString(PATHOPTION_HOSTFILE, Info.HostFile);
      WriteString(PATHOPTION_DESCRIPTION, Info.Description);
      WriteInteger(PATHOPTION_FLAGS, Info.Flags);
      Result := True;
    finally
      Close;
    end;
end;

function TOptions.ValidateName(const Name: string): string;
var
  i: Integer;
begin
  if Name = '' then
    Raise ENamedFolderError.Create(GetMsg(MNameMustNotBeEmpty))
  else begin
    Result := TNamedFolderInfo.NormalizeName(Name);
    for i := 1 to Length(Result) do
      if CharInSet(Result[i], [#0..#31, '\', ':', '*', '?', '|', '^']) then
        Raise ENamedFolderError.Create(GetMsg(MNameContainsProhibitedCharacters));
  end;
end;

end.
